--[[
	Project: dsChrono - Rift Chronograph
	Version: 1.11
	Author: Dorios
]] 

local context = UI.CreateContext("Context")
local dsChrono_frame = UI.CreateFrame("Frame", "dsChrono", context)
      dsChrono_frame.b_play = UI.CreateFrame("Texture", "dsChrono_play", dsChrono_frame)
      dsChrono_frame.b_stop = UI.CreateFrame("Texture", "dsChrono_stop", dsChrono_frame)
      dsChrono_frame.text = UI.CreateFrame("Text", "dsChrono_text", dsChrono_frame)

local active = false
local dsc_time = 0
local dsc_start = 0
local dsc_pause = 0
local dsc_play = 0
local moveable = false
local move_toggle = false
local clickOffset = {x = 0, y = 0}

local function dscFrameCreate()
	dsChrono_frame:SetPoint("TOPLEFT", context, "TOPLEFT", dsc_loc["x"], dsc_loc["y"])
      	dsChrono_frame:SetVisible(false)
      	dsChrono_frame:SetHeight(30)
      	dsChrono_frame:SetWidth(100)

      	dsChrono_frame.b_play:SetTexture("dsChrono", "dsc_play.png")
      	dsChrono_frame.b_play:SetPoint("LEFT", dsChrono_frame, "LEFT")
      	dsChrono_frame.b_play:SetPoint("CENTERY", dsChrono_frame, "CENTERY")

     	dsChrono_frame.b_stop:SetTexture("dsChrono", "dsc_stop.png")
      	dsChrono_frame.b_stop:SetPoint("LEFT", dsChrono_frame.b_play, "RIGHT")
      	dsChrono_frame.b_stop:SetPoint("CENTERY", dsChrono_frame, "CENTERY")

      	dsChrono_frame.text:SetPoint("LEFT", dsChrono_frame.b_stop, "RIGHT")
      	dsChrono_frame.text:SetPoint("CENTERY", dsChrono_frame, "CENTERY")
      	dsChrono_frame.text:SetText("00:00:00")
      	dsChrono_frame.text:SetFontSize(20)
      	dsChrono_frame.text:SetHeight(dsChrono_frame.text:GetFullHeight())
      	dsChrono_frame.text:SetWidth(dsChrono_frame.text:GetFullWidth())
      	-- dsChrono_frame:SetAlpha(80)
end

local function help()
	print("Usage: /dsc [option]:")
   	print("(none): toggle show/hide dsChrono")
   	print("move: toggle right-click to move")
   	print("help: shows dsChrono commands")	
end

local function SecToTime(Secs)
	local nSeconds = tonumber(Secs)
	if nSecs == 0 then
		--return nil;
		return "00:00:00";
	else
		local nHours = string.format("%02.f", math.floor(nSeconds/3600));
		local nMins = string.format("%02.f", math.floor(nSeconds/60 - (nHours*60)));
		local nSecs = string.format("%02.f", math.floor(nSeconds - nHours*3600 - nMins *60));
		return nHours..":"..nMins..":"..nSecs
	end
end


function dsChrono_frame.b_play.Event:LeftClick()
	if (dsc_play == 0) or (dsc_play == 2) then
		dsChrono_frame.b_play:SetTexture("dsChrono", "dsc_play.png")
		if (dsc_play == 0) then
			dsc_start = math.floor(Inspect.Time.Real())
		else
			dsc_start = dsc_start + (math.floor(Inspect.Time.Real()) - dsc_pause)
		end
		active = true
		dsc_play = 1
	else
		dsChrono_frame.b_play:SetTexture("dsChrono", "dsc_pause.png")
		active = false
		dsc_pause = math.floor(Inspect.Time.Real())
		dsc_play = 2
	end
end


function dsChrono_frame.b_stop.Event:LeftClick()
	dsChrono_frame.text:SetText("00:00:00")
	dsChrono_frame.b_play:SetTexture("dsChrono", "dsc_play.png")
	active = false
	dsc_play = 0
end

local function refresh()
	if ( (active == true) and (dsc_time < math.floor(Inspect.Time.Frame())) ) then
		dsc_time = math.floor(Inspect.Time.Real())
		dsChrono_frame.text:SetText(SecToTime(dsc_time-dsc_start))
		dsChrono_frame.text:SetHeight(dsChrono_frame.text:GetFullHeight())
		dsChrono_frame.text:SetWidth(dsChrono_frame.text:GetFullWidth())
		-- print(dsc_time.." - "..dsc_start)
	end

end

local function process(param)
	if(param == "") then 
          	if (dsChrono_frame:GetVisible() == false) then 
             		dsChrono_frame:SetVisible(true)
          	else 
             		dsChrono_frame:SetVisible(false)
			move_toggle = false 
          	end 
	elseif(param == "help") then 
         	help() 
      	elseif(param == "move") then 
          	move_toggle = not move_toggle
          	if (move_toggle) then
			if (dsChrono_frame:GetVisible() == false) then 
             			dsChrono_frame:SetVisible(true)
			end 
             		print("Move dsChrono via right-click drag")
	     		dsChrono_frame.enableMouse()  
          	else 
             		print("Moving dsChrono deactivated") 
	     		dsChrono_frame.disableMouse() 
          	end         	
      	else
        	help()
   	end
end

function dsChrono_frame.disableMouse()
	dsChrono_frame.Event["RightDown"] = nil
	dsChrono_frame.Event["RightUp"] = nil
	dsChrono_frame.Event["RightUpoutside"] = nil
	dsChrono_frame.Event["MouseMove"] = nil
end

function dsChrono_frame.enableMouse()
	dsChrono_frame.Event["RightDown"] = dsChrono_frame.RightDown
	dsChrono_frame.Event["RightUp"] = dsChrono_frame.RightUp
	dsChrono_frame.Event["RightUpoutside"] = dsChrono_frame.RightUpoutside
	dsChrono_frame.Event["MouseMove"] = dsChrono_frame.MouseMove
end

function dsChrono_frame.RightDown()
	moveable = true
	local mouseStatus = Inspect.Mouse()
	clickOffset["x"] = mouseStatus.x - dsc_loc["x"]
	clickOffset["y"] = mouseStatus.y - dsc_loc["y"]
end

function dsChrono_frame.RightUp()
	moveable = false
end

function dsChrono_frame.RightUpoutside()
	moveable = false
end

function dsChrono_frame.MouseMove()
	if moveable == true then
		local m = Inspect.Mouse()
		dsc_loc["x"] = m.x - clickOffset["x"]
		dsc_loc["y"] = m.y - clickOffset["y"]	
		dsChrono_frame:SetPoint("TOPLEFT", context, "TOPLEFT", dsc_loc["x"], dsc_loc["y"])
	end
end

local function loadSettings()
	print("dsChrono - Rift Chronograph\nType /dsc help to show commands")

	if dsc_loc == nil then
		dsc_loc = {x = UIParent:GetRight() / 2, y = UIParent:GetTop()}
	end

	dscFrameCreate()
end

table.insert(Event.Addon.Startup.End, {loadSettings, "dsChrono", "loadSettings"})
table.insert(Event.System.Update.Begin, {refresh, "dsChrono", "refresh"})
table.insert(Command.Slash.Register("dsc"), {function (params) process(params) end, "dsChrono", "Slash command"})

